@@ -13,40 +13,42 @@ class Puppet::Provider::Openstack < Puppet::Provider
   initvars # so commands will work
   commands :openstack_command => 'openstack'
 
-  # this actions are not idempotent and retries can cause
-  # duplications or endless loops
-  def self.no_retry_actions
-    %w(create remove delete)
+  @@no_retry_actions = %w(create remove delete)
+  @@command_timeout  = 40
+  # Fails on the 5th retry for a max of 212s (~3.5min) before total
+  # failure.
+  @@request_timeout  = 170
+  @@retry_sleep      = 3
+  class << self
+    [:no_retry_actions, :request_timeout, :retry_sleep].each do |m|
+      define_method m do
+        self.class_variable_get("@@#{m}")
+      end
+      define_method :"#{m}=" do |value|
+        self.class_variable_set("@@#{m}", value)
+      end
+    end
   end
 
   # timeout the openstack command
   # after this number of seconds
-  # retry the command until the request_timeout
-  def self.command_timeout
-    20
+  # retry the command until the request_timeout,
+  # unless it's a no_retry_actions call
+  def self.command_timeout(action=nil)
+    # give no_retry actions the full time limit to finish
+    return self.request_timeout() if no_retry_actions.include? action
+    self.class_variable_get("@@command_timeout")
   end
 
-  # timeout the entire request with error
-  # after this number of seconds
-  def self.request_timeout
-    60
-  end
-
-  # sleep for this number of seconds
-  # between command retries
-  def self.retry_sleep
-    3
-  end
-
-  # run the openstack command
   # with command_timeout
   def self.openstack(*args)
     begin
-      Timeout.timeout(command_timeout) do
+      action = args[1]
+      Timeout.timeout(command_timeout(action)) do
         openstack_command *args
       end
     rescue Timeout::Error
-      raise Puppet::ExecutionFailure, "Command: 'openstack #{args.inspect}' has been running for more then #{command_timeout} seconds!"
+      raise Puppet::ExecutionFailure, "Command: 'openstack #{args.inspect}' has been running for more than #{command_timeout(action)} seconds"
     end
   end
 
@@ -55,13 +57,32 @@ class Puppet::Provider::Openstack < Puppet::Provider
     Time.now.to_i
   end
 
+  def self.request_without_retry(&block)
+    previous_timeout = self.request_timeout
+    rc = nil
+    if block_given?
+      self.request_timeout = 0
+      rc = yield
+    end
+  ensure
+    self.request_timeout = previous_timeout
+    rc
+  end
+
   # Returns an array of hashes, where the keys are the downcased CSV headers
   # with underscores instead of spaces
-  def self.request(service, action, properties, credentials=nil)
+  #
+  # @param options [Hash] Other options
+  # @options :no_retry_exception_msgs [Array<Regexp>,Regexp] exception without retries
+  def self.request(service, action, properties, credentials=nil, options={})
     env = credentials ? credentials.to_env : {}
+    no_retry = options[:no_retry_exception_msgs]
+
     Puppet::Util.withenv(env) do
       rv = nil
       end_time = current_time + request_timeout
+      start_time = current_time
+      retry_count = 0
       loop do
         begin
           if action == 'list'
@@ -98,10 +119,22 @@ class Puppet::Provider::Openstack < Puppet::Provider
           break
         rescue Puppet::ExecutionFailure => exception
           raise Puppet::Error::OpenstackUnauthorizedError, 'Could not authenticate' if exception.message =~ /HTTP 40[13]/
-          raise exception if current_time > end_time
-          debug "Non-fatal error: '#{exception.message}'. Retrying for #{end_time - current_time} more seconds"
+          if current_time > end_time
+            error_message = exception.message
+            error_message += " (tried #{retry_count}, for a total of #{end_time - start_time } seconds)"
+            raise(Puppet::ExecutionFailure, error_message)
+          end
+
           raise exception if no_retry_actions.include? action
+          if no_retry
+            no_retry = [no_retry] unless no_retry.is_a?(Array)
+            no_retry.each do |nr|
+              raise exception if exception.message.match(nr)
+            end
+          end
+          debug "Non-fatal error: '#{exception.message}'. Retrying for #{end_time - current_time} more seconds"
           sleep retry_sleep
+          retry_count += 1
           retry
         end
       end
