@@ -10,17 +10,16 @@ describe 'openstacklib class' do
 
       if $::osfamily == 'RedHat' {
         class { '::openstack_extras::repo::redhat::redhat':
-          release => 'kilo',
+          release     => 'kilo',
+          manage_epel => false,
         }
-        $package_provider = 'yum'
       } else {
         include ::apt
-        $package_provider = 'apt'
       }
 
       class { '::rabbitmq':
         delete_guest_user => true,
-        package_provider  => $package_provider
+        package_provider  => $::package_provider
       }
 
       # openstacklib resources
